#include <stdlib.h>
#include <algorithm>
#include <string.h>
#include <iostream>
#include "measure.hh"


// fill array with random values from [0,1]
void fillData(int* aArray, int aSize) {
  for (int i = 0; i < aSize; ++i) {
    aArray[i] = rand() % 2;   
  }
}


// fill index array sequentially
void fillIndex(int* aArray, int aSize) {
  for (int i = 0; i < aSize; ++i) {
    aArray[i] = i;
  }
}


int main(int argc, char* argv[]) {
  // random access?
  bool lRand = false;
  // array size 
  int lSize = 1'000'000'000;

  if (argc > 1) {
    for (int i = 1; i < argc; ++i) {
      if (!strcmp(argv[i], "-r")) {
        lRand = true;
      }
      
      if (!strcmp(argv[i], "-n")) {
        lSize = atol(argv[++i]);
      }
    }
  }

  int *lData  = new int[lSize];
  int *lIndex = new int[lSize];
 
  fillData(lData, lSize);
  fillIndex(lIndex, lSize);

  // shuffle index array to achieve random memory access
  if (lRand) {
    std::random_shuffle(lIndex, lIndex + lSize);
  }

  int lSum = 0;

  // add up all numbers in array, access array sequentially or randomly
  Measure lMeasure;
  lMeasure.start();
  for (int i = 0; i < lSize; ++i) {
    lSum += lData[lIndex[i]]; 
  }
  lMeasure.stop();

  // print runtime
  std::cout << (lRand ? "Random Access: " : "Sequential Access: ") << std::endl;
  std::cout << "Time: " << lMeasure.mCpuTime() << "s" << std::endl;

  delete[] lData;
  delete[] lIndex;

  return 0;
}
